<?php

namespace Drupal\ckeditor_image\Plugin\CKEditorPlugin;

use Drupal\ckeditor\CKEditorPluginBase;
use Drupal\ckeditor\CKEditorPluginConfigurableInterface;
use Drupal\ckeditor\CKEditorPluginContextualInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\editor\Entity\Editor;

/**
 * Defines the "CKEditor Quick Image Uploader" plugin.
 *
 * @CKEditorPlugin(
 *   id = "jsplus_image",
 *   label = @Translation("CKEditor Quick Image Uploader"),
 *   module = "ckeditor_image"
 * )
 */
class ckeditor_image extends CKEditorPluginBase implements CKEditorPluginConfigurableInterface, CKEditorPluginContextualInterface {

  public $plugins = array(
     'jsplus_image' => array(
    'buttons' => array(array('label' => 'Quick Image Uploader')),
    'urlDoc' => 'http://js.plus/products/quick-image-uploader/configuration',
    'params' => array(
        array(
            'name' => 'jsplus_image_uploader',
            'default' => '/libraries/jsplus_uploader/uploader.php',
            'type' => 'str',
            'order' => 1000,
            'title' => 'Uploader URL',
            'hint' => 'URL to uploader can be set manually if you have custom URL routing',
            'widget' => 'text'
        ),
        array(
            'name' => 'jsplus_image_template',
            'default' => '<img src="{IMAGE}"/>',
            'type' => 'str',
            'order' => 1010,
            'title' => 'Template',
            'hint' => 'HTML template (one root tag only) used for image insertion, available substitutions are: {IMAGE}, {FILENAME}.',
            'widget' => 'textarea'
        ),
        array(
            'name' => 'jsplus_image_default_img_resize',
            'default' => TRUE,
            'type' => 'bool',
            'order' => 1020,
            'title' => 'Resize image by default',
            'hint' => 'Default state of image resize checkbox',
            'widget' => 'checkbox'
        ),
        array(
            'name' => 'jsplus_image_default_img_resize_width',
            'default' => 800,
            'type' => 'int',
            'order' => 1030,
            'title' => 'Width for uploaded images by default',
            'hint' => '0 means keeping original width',
            'widget' => 'text'
        ),
        array(
            'name' => 'jsplus_image_default_img_resize_height',
            'default' => 600,
            'type' => 'int',
            'order' => 1040,
            'title' => 'Height for uploaded images by default',
            'hint' => '0 means keeping original height',
            'widget' => 'text'
        ),
        array(
            'name' => 'jsplus_image_default_img_resize_enlarge',
            'default' => FALSE,
            'type' => 'bool',
            'order' => 1050,
            'title' => 'Enlarge image if too small by default',
            'hint' => 'Default state of enlarge option',
            'widget' => 'checkbox'
        ),
        array(
            'name' => 'jsplus_image_img_resize_show',
            'default' => TRUE,
            'type' => 'bool',
            'order' => 1060,
            'title' => 'Resize panel is visible',
            'hint' => 'Show or hide panel with image resize options',
            'widget' => 'checkbox'
        ),
        array(
            'name' => 'jsplus_image_allowed_ext',
            'default' => 'jpg,jpeg,gif,png,bmp,tif,tiff',
            'type' => 'str',
            'order' => 1070,
            'title' => 'Allowed file extensions',
            'hint' => 'The list with allowed file extensions, comma separated',
            'widget' => 'text'
        )
    )
)
  );

    /**
     * {@inheritdoc}
     */
    public function getButtons() {
        if (!$this->isInstalled())
            return array();

        $buttons = array();
        foreach ($this->plugins as $pluginName => $pluginDef) {

            if (isset($pluginDef['buttons'])) {
                foreach($pluginDef['buttons'] as $buttonName => $buttonDef) {
                    $image = '/icons/' . $pluginName . '.png';
                    if (isset($buttonDef['image']))
                        $image = $pluginDef['image'];
                    $button = array(
                        'label' => $buttonDef['label'],
                        'image' => '/libraries/jsplus_image' . $image
                    );
                    $buttons[$pluginName] = $button;
                }
            }

        }
        return $buttons;
    }

  /**
   * {@inheritdoc}
   */
  public function getFile() {
    return '/libraries/jsplus_image/plugin.js';
  }

  /**
   * {@inheritdoc}
   */
  public function getDependencies(Editor $editor) {
    return array();
  }

  /**
   * {@inheritdoc}
   */
  public function getLibraries(Editor $editor) {
    return array();
  }

  /**
   * {@inheritdoc}
   */
  public function isInternal() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function isEnabled(Editor $editor) {
    return TRUE;
  }

  function getConfigParam($settings, $param, $default, $type, $inside) {
    $name = $param;
    if ($inside != null)
        $name = $inside . '.' . $name;
    if (isset($settings[$name]) && strlen($settings[$name]) > 0)
        $value = $settings[$name];
    else
        $value = $default;
    if (isset($type) && $type == 'int') {
        $value = intval($value);
    } else if (isset($type) && $type == 'bool') {
        if ($value == '1')
            $value = true;
        else if ($value == '0')
            $value = false;
    } else if (isset($type) && $type == 'json') {
        $value = json_decode($value);
    }
    return $value;
  }

  /**
   * {@inheritdoc}
   */
  public function getConfig(Editor $editor) {
    $settings = $editor->getSettings()['plugins']['jsplus_image'];
    $result = array();
    foreach ($this->plugins as $pluginName => $pluginDef) {
        if (isset($pluginDef['params'])) {
            foreach ($pluginDef['params'] as $paramDef) {
                $value = $this->getConfigParam($settings, $paramDef['name'], $paramDef['default'], $paramDef['type'], isset($paramDef['inside']) ? $paramDef['inside'] : null);
                $key = null;
                if (!isset($pluginDef['inside']))
                    $key = $paramDef['name'];
                else {
                    if (!isset($result[$paramDef['inside']]))
                        $result[$paramDef['inside']] = array();
                    $key = $paramDef['inside'][$paramDef['name']];
                }
                $result[$key] = $value;
            }
        }
    }

    return $result;
  }

  function addSelectToForm(& $form, $settings, $param, $title, $default, $options, $inside, $urlHelp) {
      $form[$param] = array(
        '#type' => 'select',
        '#title' => $title,
        '#options' => $options,
        '#default_value' => $this->getConfigParam($settings, $param, $default, 'str', $inside),
        '#attributes' => array('data-url-help' => $urlHelp == '' ? '' : ($urlHelp . '#' . $param), 'data-param-name' => $param)
      );
  }

  function addTextboxToForm(& $form, $settings, $param, $title, $default, $desc, $inside, $urlHelp) {
        $form[$param] = array(
          '#type' => 'textfield',
          '#title' => $title,
          '#default_value' => $this->getConfigParam($settings, $param, $default, 'str', $inside),
          '#description' => $desc,
          '#attributes' => array('data-url-help' => $urlHelp == '' ? '' : ($urlHelp . '#' . $param), 'data-param-name' => $param)
        );
  }

  function addTextareaToForm(& $form, $settings, $param, $title, $default, $desc, $inside, $urlHelp) {
        $form[$param] = array(
          '#type' => 'textarea',
          '#title' => $title,
          '#default_value' => $this->getConfigParam($settings, $param, $default, 'str', $inside),
          '#description' => $desc,
          '#attributes' => array('data-url-help' => $urlHelp == '' ? '' : ($urlHelp . '#' . $param), 'data-param-name' => $param)
        );
  }

  function addCheckboxToForm(& $form, $settings, $param, $title, $default, $desc, $inside, $urlHelp) {
        $form[$param] = array(
          '#type' => 'checkbox',
          '#title' => $title,
          '#default_value' => $this->getConfigParam($settings, $param, $default, 'bool', $inside),
          '#description' => $desc,
          '#attributes' => array('data-url-help' => $urlHelp == '' ? '' : ($urlHelp . '#' . $param), 'data-param-name' => $param)
        );
  }

  public function isInstalled() {
    return file_exists($_SERVER['DOCUMENT_ROOT'] . '/libraries/jsplus_image');
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state, Editor $editor) {
    $editor_settings = $editor->getSettings();
    if (isset($editor_settings['plugins']['jsplus_image'])) {
      $settings = $editor_settings['plugins']['jsplus_image'];
    }

    $form['#attached']['library'][] = 'ckeditor_image/ckeditor_image.admin';

    if (!$this->isInstalled()) {
        $form['warning'] = array(
            '#markup' => 'Looks like CKEditor Quick Image Uploader (Drupal 8 module) is installed but CKEditor add-ons not found. In order to use this module please copy CKEditor add-ons into "libraries" forder in the root of your Drupal 8 installation (create the directory if it does not exist).'
        );
        return $form;
    }

    $params = array();
    foreach ($this->plugins as $pluginName => $pluginDef) {
        if (isset($pluginDef['params']))
            foreach ($pluginDef['params'] as $paramDef) {
                $paramDef['urlDoc'] = $pluginDef['urlDoc'];
                $params[$paramDef['order']] = $paramDef;
            }
    }
    ksort($params); // sort by key (order)

    foreach ($params as $order => $paramDef) {
        $inside = isset($paramDef['inside']) ? $paramDef['inside'] : null;
        if ($paramDef['widget'] == 'select') {
            $this->addSelectToForm($form, $settings, $paramDef['name'], t($paramDef['title']), $paramDef['default'], $paramDef['widgetOptions'], $inside, $paramDef['urlDoc']);
        } else if ($paramDef['widget'] == 'checkbox') {
            $this->addCheckboxToForm($form, $settings, $paramDef['name'], t($paramDef['title']), $paramDef['default'], $paramDef['hint'], $inside, $paramDef['urlDoc']);
        } else if ($paramDef['widget'] == 'textarea') {
            $this->addTextareaToForm($form, $settings, $paramDef['name'], t($paramDef['title']), $paramDef['default'], $paramDef['hint'], $inside, $paramDef['urlDoc']);
        } else {
            $this->addTextboxToForm($form, $settings, $paramDef['name'], t($paramDef['title']), $paramDef['default'], $paramDef['hint'], $inside, $paramDef['urlDoc']);
        }
    }

    return $form;
  }

}
